; ______________________________________________________________________
;
;                     BLOW the EEPROM in a PIC16C84
;                     Mark J Cox, Jan-May 1994
;
;                     EMAIL:m.j.cox@bradford.ac.uk
;                     URL:http://www.eia.brad.ac.uk/mark.html
; ______________________________________________________________________
;
; Legal:
;
; This file may be modified and distributed freely as
; long as my Name and Email address remain intact.  This program
; is experimental; so you can't blame me if it doesn't work, causes
; your PIC chips to blow up, your dog to run away or anything else
; consequential.  If you don't like this then don't bother using
; this program.  Report bugs, changes etc. to above email address. If
; you make any modifications then send me a copy.  Information from
; "Microchip Data Book 1993".  Microchip Technology Inc.  2355 West
; Chandler Blvd. Chandler. AZ 85224-6199 USA.  Tel 602-786-7200
; ______________________________________________________________________
;
; History:
;
; v1.00 MJC 13.01.94 Handles Program Memory, Config memory, reads a file
; v1.01 MJC 14.01.94 Timing for 486-33 (about 2-3 X as slow as it needs)
; v1.02 MJC 08.05.94 Added Schematic and now gets port address
; v1.10 MJC 09.05.94 First release
;
;   1. Does not handle addresses out of order - program crashes
;   2. Won't read back or verify the EEPROM.  You'd need to add a chip
;   3. Won't program the Data Memory.  Fix this if you want
;   4. Computers >486-33 might not work.  Switch off turbo mode!
;
; This programmer was designed to be as cheap and simple as possible.
; Add switching power supplies and bi-directional stuff and it starts
; costing more money!
; ______________________________________________________________________
;
; Schematic:
; (15 or 14?) red +5v
;                                                       PC Power Supply
;                                PIC16C84              
;    Parallel Port Pin          Ŀ     Switch /o GND
;                   (2)  D0  Ĵ13     4o/ o +12V
;                   (3)  D1  Ĵ12    14Ŀ    
;                                  5  16ı +5V
;                                   3K Resistor (*)
;                                         22pF Capacitor (*)
;                   (25) GND  GND
;
; *=Value not critical.  The switch is a SPDT type, Pin 4 is connected
; to either 0v or +12v depending on the switch position.  Get the +12 and
; +5v from inside your computer (if you dare) or a spare old power 
; supply.  PC power supplies are pretty tolerant of shorting - but dont
; blame me if you short it out and have to buy a new one!
; ______________________________________________________________________
; 
; Sample BLOW.DAT file:
;
; 0000 1683			<- Each line must be EXACTLY this 
; 0001 0186                        format and not have any extra
; 0002 1283                        spaces or returns.  You'll have
; 0003 30FF                        to work out the opcodes yourself!
; 0004 0086
; 0005 0064
; 0006 2105
; ______________________________________________________________________
;
; Assembler:
;
; I've only ever compiled this in A86.  Its a shareware assembler I
; recommend you all register if you use.  It will probably be trivial
; to make it work with TASM, MASM etc.
;

jmp main

; Output 1 = Clock, 0 = Data
OUTPUTPORT	dw 0378

; --- Equ's for Write Command ---
WR_LDCONFIG	equ 0
WR_LDPROG	equ 2
WR_RDPROG	equ 4
WR_INCADDR	equ 6
WR_BEGINPROG	equ 8
WR_LDDATA	equ 3
WR_RDDATA	equ 5

; Main:
; 	location = 0
;	repeat
; 		Get data into AX
; 		call Program_Cycle
;		call inc_address
; 	end

WRITELN macro
	push ax
	mov dx,offset #1
	mov ah,9
	int 021
	pop ax
	#em

msg_intro: 
db '',13,10
db ' PIC16C84 Programmer v1.10                          Copyr. Mark J Cox 1994',13,10
db '',13,10
db 13,10
db 'Make sure LPT1 is connected then switch on 12v supply',13,10
db 'Press G to commence programming from file BLOW.DAT, or ESC to end$'

msg_outro: 
db 13,10,13,10
db '',13,10
db 'Programming Finished, switch off 12v supply',13,10
db '$'

LPT1	equ 8
LPT2	equ 12

main:		mov ax,040
		mov es,ax
		mov ax,es:[LPT1]		; LPT1 IO address
		mov [OUTPUTPORT],ax
		mov es,cs

		mov dx,[OUTPUTPORT]
		mov al,0		; R6 and R7 = 0
		out dx,al
		WRITELN msg_intro
PL1:		mov ax,0
		int 016
		cmp al,27
		jz PL2
		cmp al,'g'
		jz PL3
		cmp al,'G'
		jnz PL1
PL3:		call process_file
PL2:		WRITELN msg_outro
		mov ax,4c00
		int 021


; ----------------------------------------------------------------------
; Process file
;
; Open file BLOW.DAT, read 11 bytes (0000 0000\n\r) and process 
; until EOF
; ----------------------------------------------------------------------

filename:	db 'BLOW.DAT',0
fhandle:	dw ?
msg_cantopen:	db 13,10,13,10,'*** FATAL: Could not open input file',13,10,'$'
buff_11b:	db '...........$'
cloc:		dw ?		; Current programming location

process_file:	mov w[cloc],0
		mov dx,offset filename
		mov ax,03d00
		int 021
		jnc QL1
		WRITELN msg_cantopen
		ret		
QL1:		mov [fhandle],ax
		mov bx,ax
QL3:
		; read 11 bytes into buf_11b

		mov cx,11
		mov dx,offset buff_11b
		mov ah,03f
		int 021
		jc QL2
		sub ax,cx		; Not 11 bytes!
		jnz QL2
		call process_buffer
		jmp QL3
QL2:		mov bx,[fhandle]
		mov ah,03e
		int 021
		clc
		ret


; ----------------------------------------------------------------------
; Process Buffer
; 
; Its read in a 11 byte line, handle it here

; ----------------------------------------------------------------------
; if l=>r then no carry, if l<r then carry

tmp_msg:	db ' (now jump to config mem)$'

process_buffer:	pusha
		mov dx,offset buff_11b
		call ascii2num
		cmp ax,02000		; Have we reached special area?
		jc RL1
		mov bx,[cloc]
		cmp bx,02000
		jnc RL1
		WRITELN tmp_msg		; Do special 2000 select here
		call goto_2000
		mov bx,02000
		mov [cloc],bx
RL1:		cmp ax,[cloc]		; Is it what we expect?
		jz RL2
		inc w[cloc]
		call inc_address		
		jmp RL1

RL2:		mov dx,offset buff_11b+5
		call ascii2num
		call program_cycle
		inc w[cloc]
		call inc_address
		popa
		ret


; ----------------------------------------------------------------------
; Ascii to num
;
; DX points to ascii "01F2" convert and store in AX
; ----------------------------------------------------------------------


ascii2num:	mov di,dx
		mov cx,4
		xor ax,ax
SL1:		shl ax,1
		shl ax,1
		shl ax,1
		shl ax,1
		mov bl,[di]
		xor bh,bh
		inc di
		cmp bl,'9'+1
		jnc SL2
		sub bl,'0'
		add ax,bx
		jmp SL3
SL2:		sub bl,'A'-10
		add ax,bx
SL3:		loop SL1
		ret		

; ----------------------------------------------------------------------
; Program Cycle
;
; Program current location, where AX is 14 bit data (in 14 lsb's)
; ----------------------------------------------------------------------

msg_progloc:	db 13,10,13,10,' Programming Location  $'
msg_with:	db 'with $'
msg_end:	db '',13,10,'$'

program_cycle:	push ax
		WRITELN msg_progloc
		mov ax,[cloc]
		call display_hex
		pop ax
		WRITELN msg_with
		call display_hex
		WRITELN msg_end
		mov bl,WR_LDPROG
		call write_command
		mov bx,ax
		call write_data
		mov bx,WR_BEGINPROG
		call write_command
		call delay_10ms
		ret

; ----------------------------------------------------------------------
; Increment Address for Programming
; ----------------------------------------------------------------------

inc_address:	mov bl,WR_INCADDR
		call write_command
		ret

goto_2000:	mov bl,WR_LDCONFIG
		call write_command
		ret

; ----------------------------------------------------------------------
; Write Command
;
; From lower 6 bits of BL, lsb first.  All registers preseved
; ----------------------------------------------------------------------

msg_writecmd:	db 13,10,'  Cmd  $'

write_command:	push ax,bx,cx,dx
		WRITELN msg_writecmd
		mov al,bl
		xor ah,ah
		call display_hex
		mov cx,6
		mov dx,cs:[OUTPUTPORT]
TL1:		ror bl,1		;Shr through carry. carry=lsb
		mov al,2
		IF C mov al,3
		out dx,al		; Clk =1, Data set
		call delay_100nS
		sub al,2		; Clk =0, Data set
		out dx,al
		call display_hex1
		call delay_100nS
		loop TL1
		call delay_1uS
		pop dx,cx,bx,ax
		ret


; ----------------------------------------------------------------------
; Write Data
;
; From lower 14 bits of BX, lsb first.  All registers preseved
; Outputs the initial start and final stop (=0) bits
; ----------------------------------------------------------------------

msg_writedata: db 13,10,'  Data $'

write_data:	push ax,bx,cx,dx
		WRITELN msg_writedata
		mov ax,bx
		call display_hex
		mov cx,16
		shl bx,1		; 0-14 bits-0
		and bx,07ffe		; Clear all but the 14 bits

		mov dx,cs:[OUTPUTPORT]
UL1:		ror bx,1		;Shr through carry. carry=lsb
		mov al,2
		IF C mov al,3
		out dx,al		; Clk =1, Data set
		call delay_100nS
		sub al,2		; Clk =0, Data set
		call display_hex1
		out dx,al
		call delay_100nS
		loop UL1
		call delay_1uS
		pop dx,cx,bx,ax
		ret

; ----------------------------------------------------------------------
; Delay for 1uS 
; ----------------------------------------------------------------------

delay_1us:	
		push cx
		mov cx,3
VL1:		nop
		nop
		loop VL1
		pop cx
		ret

; ----------------------------------------------------------------------
; Delay for 100ns 
; ----------------------------------------------------------------------

; This is >>100nS

delay_100ns:	
		push cx
		mov cx,10
WL1:		nop
		nop
		loop WL1
		pop cx
		ret

; ----------------------------------------------------------------------
; Delay for 10ms
; ----------------------------------------------------------------------

delay_10ms:	push cx
		mov cx,0
XL1:		nop
		nop
		loop XL1
		pop cx
		ret

hex_buf: db '---- $'
s_buf: db '-$'

display_hex1:
	pusha
	mov di,offset s_buf
	and al,0f
	call dig_to_hex
	mov [di],al
	mov dx,offset s_buf
	mov ah,09
	int 021
	popa
	ret

display_hex:
	pusha
	mov di,offset hex_buf+3
	mov cx,4
YL1:	push ax
	and al,0f
	call dig_to_hex
	mov [di],al
	dec di
	pop ax
	shr ax,1
	shr ax,1
	shr ax,1
	shr ax,1
	loop YL1
	mov dx,offset hex_buf
	mov ah,9
	int 021
	popa
	ret

dig_to_hex:	; In al digit, out al = 0-9 A-H
	add al,'0'
	cmp al,'9'+1
	jc ZL1
	sub al,'0'+10
	add al,'A'
ZL1:	ret
